@@ -22,14 +22,57 @@
 #ifndef GEANY_KEYBINDINGS_H
 #define GEANY_KEYBINDINGS_H 1
 
+#include "gtkcompat.h"
+
 G_BEGIN_DECLS
 
+/** Defines the primary modifier mask which is the Ctrl key mask on
+ * UNIX/Windows and Command key mask on OS X. When testing for the mask
+ * presence, use together with keybindings_get_modifiers() which adds 
+ * @c GEANY_PRIMARY_MOD_MASK when needed.
+ * @since 1.25. */
+#ifdef __APPLE__
+#define GEANY_PRIMARY_MOD_MASK GDK_META_MASK
+#else
+#define GEANY_PRIMARY_MOD_MASK GDK_CONTROL_MASK
+#endif
+
+/** A collection of keybindings grouped together. */
+typedef struct GeanyKeyGroup GeanyKeyGroup;
+typedef struct GeanyKeyBinding GeanyKeyBinding;
+
+/** Function pointer type used for keybinding group callbacks.
+ *
+ * You should return @c TRUE to indicate handling the callback. (Occasionally, if the keybinding
+ * cannot apply in the current situation, it is useful to return @c FALSE to allow a later keybinding
+ * with the same key combination to handle it). */
+typedef gboolean (*GeanyKeyGroupCallback) (guint key_id);
+
+/** Function pointer type used for keybinding group callbacks, with userdata for passing context.
+ *
+ * You should return @c TRUE to indicate handling the callback. (Occasionally, if the keybinding
+ * cannot apply in the current situation, it is useful to return @c FALSE to allow a later keybinding
+ * with the same key combination to handle it).
+ *
+ * @since 1.26 (API 226) */
+typedef gboolean (*GeanyKeyGroupFunc)(GeanyKeyGroup *group, guint key_id, gpointer user_data);
+
 /** Function pointer type used for keybinding callbacks. */
 typedef void (*GeanyKeyCallback) (guint key_id);
 
+/** Function pointer type used for keybinding callbacks, with userdata for passing context
+ *
+ * You should return @c TRUE to indicate handling the callback. (Occasionally, if the keybinding
+ * cannot apply in the current situation, it is useful to return @c FALSE to allow a later keybinding
+ * with the same key combination to handle it).
+ *
+ * @since 1.26 (API 226) */
+typedef gboolean (*GeanyKeyBindingFunc)(GeanyKeyBinding *key, guint key_id, gpointer user_data);
+
 /** Represents a single keybinding action.
+ *
  * Use keybindings_set_item() to set. */
-typedef struct GeanyKeyBinding
+struct GeanyKeyBinding
 {
 	guint key;				/**< Key value in lower-case, such as @c GDK_a or 0 */
 	GdkModifierType mods;	/**< Modifier keys, such as @c GDK_CONTROL_MASK or 0 */
@@ -44,36 +87,10 @@ typedef struct GeanyKeyBinding
 	guint id;
 	guint default_key;
 	GdkModifierType default_mods;
-}
-GeanyKeyBinding;
-
-
-/** Function pointer type used for keybinding group callbacks.
- * You should return @c TRUE to indicate handling the callback. (Occasionally, if the keybinding
- * cannot apply in the current situation, it is useful to return @c FALSE to allow a later keybinding
- * with the same key combination to handle it). */
-typedef gboolean (*GeanyKeyGroupCallback) (guint key_id);
-
-/** A collection of keybindings grouped together. */
-typedef struct GeanyKeyGroup GeanyKeyGroup;
-
-/* Plugins should not set these fields. */
-#ifdef GEANY_PRIVATE
-struct GeanyKeyGroup
-{
-	const gchar *name;		/* Group name used in the configuration file, such as @c "html_chars" */
-	const gchar *label;		/* Group label used in the preferences dialog keybindings tab */
-	GeanyKeyGroupCallback callback;	/* use this or individual keybinding callbacks */
-	gboolean plugin;		/* used by plugin */
-	GPtrArray *key_items;	/* pointers to GeanyKeyBinding structs */
-	gsize plugin_key_count;			/* number of keybindings the group holds */
-	GeanyKeyBinding *plugin_keys;	/* array of GeanyKeyBinding structs */
+	GeanyKeyBindingFunc cb_func;
+	gpointer cb_data;
+	GDestroyNotify cb_data_destroy;
 };
-#endif
-
-
-extern GPtrArray *keybinding_groups;	/* array of GeanyKeyGroup pointers */
-
 
 /* Note: we don't need to break the plugin ABI when appending keybinding or keygroup IDs,
  * just make sure to insert immediately before the _COUNT item, so
@@ -152,7 +169,8 @@ enum GeanyKeyBindingID
 	GEANY_KEYS_FORMAT_SENDTOVTE,				/**< Keybinding. */
 	GEANY_KEYS_PROJECT_PROPERTIES,				/**< Keybinding. */
 	GEANY_KEYS_DOCUMENT_LINEWRAP,				/**< Keybinding. */
-	GEANY_KEYS_EDITOR_MACROLIST,				/**< Keybinding. */
+	GEANY_KEYS_EDITOR_MACROLIST,				/**< Keybinding.
+												 * @deprecated 1.25, it doesn't do anything anymore */
 	GEANY_KEYS_EDITOR_SUPPRESSSNIPPETCOMPLETION, /**< Keybinding. */
 	GEANY_KEYS_FOCUS_SIDEBAR_SYMBOL_LIST,		/**< Keybinding. */
 	GEANY_KEYS_GOTO_LINESTART,					/**< Keybinding. */
@@ -248,14 +266,44 @@ enum GeanyKeyBindingID
 	GEANY_KEYS_GOTO_LINESTARTVISUAL,			/**< Keybinding. */
 	GEANY_KEYS_DOCUMENT_CLONE,					/**< Keybinding. */
 	GEANY_KEYS_FILE_QUIT,						/**< Keybinding. */
+	GEANY_KEYS_FILE_PROPERTIES,					/**< Keybinding. */
+	GEANY_KEYS_FORMAT_SENDTOCMD4,				/**< Keybinding. */
+	GEANY_KEYS_FORMAT_SENDTOCMD5,				/**< Keybinding. */
+	GEANY_KEYS_FORMAT_SENDTOCMD6,				/**< Keybinding. */
+	GEANY_KEYS_FORMAT_SENDTOCMD7,				/**< Keybinding. */
+	GEANY_KEYS_FORMAT_SENDTOCMD8,				/**< Keybinding. */
+	GEANY_KEYS_FORMAT_SENDTOCMD9,				/**< Keybinding. */
+	GEANY_KEYS_EDITOR_DELETELINETOBEGINNING,	/**< Keybinding. */
+	GEANY_KEYS_DOCUMENT_STRIPTRAILINGSPACES,	/**< Keybinding. 
+												 * @since 1.34 (API 238) */
 	GEANY_KEYS_COUNT	/* must not be used by plugins */
 };
 
 
-void keybindings_init(void);
+void keybindings_send_command(guint group_id, guint key_id);
+
+GeanyKeyBinding *keybindings_set_item(GeanyKeyGroup *group, gsize key_id,
+		GeanyKeyCallback callback, guint key, GdkModifierType mod,
+		const gchar *name, const gchar *label, GtkWidget *menu_item);
+
+GeanyKeyBinding *keybindings_set_item_full(GeanyKeyGroup *group, gsize key_id,
+		guint key, GdkModifierType mod, const gchar *kf_name, const gchar *label,
+		GtkWidget *menu_item, GeanyKeyBindingFunc func, gpointer pdata,
+		GDestroyNotify destroy_notify);
+
+GeanyKeyBinding *keybindings_get_item(GeanyKeyGroup *group, gsize key_id);
+
+GdkModifierType keybindings_get_modifiers(GdkModifierType mods);
 
 void keybindings_load_keyfile(void);
 
+#ifdef GEANY_PRIVATE
+
+extern GPtrArray *keybinding_groups;	/* array of GeanyKeyGroup pointers */
+
+
+void keybindings_init(void);
+
 void keybindings_free(void);
 
 GeanyKeyGroup *keybindings_get_core_group(guint id);
@@ -265,18 +313,10 @@ GeanyKeyGroup *keybindings_set_group(GeanyKeyGroup *group, const gchar *section_
 
 void keybindings_free_group(GeanyKeyGroup *group);
 
-GeanyKeyBinding *keybindings_set_item(GeanyKeyGroup *group, gsize key_id,
-		GeanyKeyCallback callback, guint key, GdkModifierType mod,
-		const gchar *name, const gchar *label, GtkWidget *menu_item);
-
-GeanyKeyBinding *keybindings_get_item(GeanyKeyGroup *group, gsize key_id);
-
 gchar *keybindings_get_label(GeanyKeyBinding *kb);
 
 void keybindings_update_combo(GeanyKeyBinding *kb, guint key, GdkModifierType mods);
 
-void keybindings_send_command(guint group_id, guint key_id);
-
 GeanyKeyBinding *keybindings_lookup_item(guint group_id, guint key_id);
 
 /* just write the content of the keys array to the config file */
@@ -288,6 +328,8 @@ gboolean keybindings_check_event(GdkEventKey *ev, GeanyKeyBinding *kb);
 
 void keybindings_dialog_show_prefs_scroll(const gchar *name);
 
+#endif /* GEANY_PRIVATE */
+
 G_END_DECLS
 
-#endif
+#endif /* GEANY_KEYBINDINGS_H */
