@@ -30,47 +30,14 @@
 */
 
 #include "tm_source_file.h"
+#include "tm_parser.h"
 #include <glib-object.h>
 
-#ifdef __cplusplus
-extern "C"
-{
-#endif
+G_BEGIN_DECLS
 
 /** Use the TM_TAG() macro to cast a pointer to (TMTag *) */
 #define TM_TAG(tag) ((TMTag *) tag)
 
-/**
- Types of tags. It is a bitmask so that multiple tag types can
- be used simultaneously by 'OR'-ing them bitwise.
- e.g. tm_tag_class_t | tm_tag_struct_t
-*/
-typedef enum
-{
-	tm_tag_undef_t = 0, /**< Unknown type */
-	tm_tag_class_t = 1, /**< Class declaration */
-	tm_tag_enum_t = 2, /**< Enum declaration */
-	tm_tag_enumerator_t = 4, /**< Enumerator value */
-	tm_tag_field_t = 8, /**< Field (Java only) */
-	tm_tag_function_t = 16, /**< Function definition */
-	tm_tag_interface_t = 32, /**< Interface (Java only) */
-	tm_tag_member_t = 64, /**< Member variable of class/struct */
-	tm_tag_method_t = 128, /**< Class method (Java only) */
-	tm_tag_namespace_t = 256, /**< Namespace declaration */
-	tm_tag_package_t = 512, /**< Package (Java only) */
-	tm_tag_prototype_t = 1024, /**< Function prototype */
-	tm_tag_struct_t = 2048, /**< Struct declaration */
-	tm_tag_typedef_t = 4096, /**< Typedef */
-	tm_tag_union_t = 8192, /**< Union */
-	tm_tag_variable_t = 16384, /**< Variable */
-	tm_tag_externvar_t = 32768, /**< Extern or forward declaration */
-	tm_tag_macro_t = 65536, /**<  Macro (without arguments) */
-	tm_tag_macro_with_arg_t = 131072, /**< Parameterized macro */
-	tm_tag_file_t = 262144, /**< File (Pseudo tag) - obsolete */
-	tm_tag_other_t = 524288, /**< Other (non C/C++/Java tag) */
-	tm_tag_max_t = 1048575 /**< Maximum value of TMTagType */
-} TMTagType;
-
 /**
  Tag Attributes. Note that some attributes are available to file
  pseudotags only. Attributes are useful for specifying as arguments
@@ -114,13 +81,9 @@ typedef enum
 #define TAG_IMPL_UNKNOWN 'x' /**< Unknown implementation */
 
 /**
- This structure holds all information about a tag, including the file
- pseudo tag. It should always be created indirectly with one of the tag
- creation functions such as tm_source_file_parse() or tm_tag_new_from_file().
- Once created, they can be sorted, deduped, etc. using functions such as
- tm_tags_sort() or tm_tags_dedup()
-*/
-typedef struct _TMTag
+ * The TMTag structure represents a single tag in the tag manager.
+ **/
+typedef struct TMTag
 {
 	char *name; /**< Name of tag */
 	TMTagType type; /**< Tag Type */
@@ -137,49 +100,34 @@ typedef struct _TMTag
 	char *var_type; /**< Variable type (maps to struct for typedefs) */
 	char access; /**< Access type (public/protected/private/etc.) */
 	char impl; /**< Implementation (e.g. virtual) */
-	langType lang; /**< Programming language of the file */
+	TMParserType lang; /* Programming language of the file */
 } TMTag;
 
-
-#ifdef GEANY_PRIVATE
-
-extern const TMTagType TM_GLOBAL_TYPE_MASK;
-
-
-typedef enum {
-	TM_FILE_FORMAT_TAGMANAGER,
-	TM_FILE_FORMAT_PIPE,
-	TM_FILE_FORMAT_CTAGS
-} TMFileFormat;
-
-
 /* The GType for a TMTag */
 #define TM_TYPE_TAG (tm_tag_get_type())
 
 GType tm_tag_get_type(void) G_GNUC_CONST;
 
-TMTag *tm_tag_new(TMSourceFile *file, const tagEntryInfo *tag_entry);
+#ifdef GEANY_PRIVATE
 
-TMTag *tm_tag_new_from_file(TMSourceFile *file, FILE *fp, gint mode, TMFileFormat format);
-
-gboolean tm_tag_write(TMTag *tag, FILE *file, guint attrs);
+TMTag *tm_tag_new(void);
 
 void tm_tags_remove_file_tags(TMSourceFile *source_file, GPtrArray *tags_array);
 
 GPtrArray *tm_tags_merge(GPtrArray *big_array, GPtrArray *small_array, 
 	TMTagAttrType *sort_attributes, gboolean unref_duplicates);
 
-gboolean tm_tags_sort(GPtrArray *tags_array, TMTagAttrType *sort_attributes, 
+void tm_tags_sort(GPtrArray *tags_array, TMTagAttrType *sort_attributes,
 	gboolean dedup, gboolean unref_duplicates);
 
 GPtrArray *tm_tags_extract(GPtrArray *tags_array, guint tag_types);
 
-gboolean tm_tags_prune(GPtrArray *tags_array);
+void tm_tags_prune(GPtrArray *tags_array);
 
-gboolean tm_tags_dedup(GPtrArray *tags_array, TMTagAttrType *sort_attributes, gboolean unref_duplicates);
+void tm_tags_dedup(GPtrArray *tags_array, TMTagAttrType *sort_attributes, gboolean unref_duplicates);
 
 TMTag **tm_tags_find(const GPtrArray *tags_array, const char *name,
-		gboolean partial, gboolean tags_array_sorted, guint * tagCount);
+		gboolean partial, guint * tagCount);
 
 void tm_tags_array_free(GPtrArray *tags_array, gboolean free_all);
 
@@ -191,6 +139,8 @@ TMTag *tm_tag_ref(TMTag *tag);
 
 gboolean tm_tags_equal(const TMTag *a, const TMTag *b);
 
+gboolean tm_tag_is_anon(const TMTag *tag);
+
 #ifdef TM_DEBUG /* various debugging functions */
 
 const char *tm_tag_type_name(const TMTag *tag);
@@ -207,8 +157,6 @@ gint tm_tag_scope_depth(const TMTag *t);
 
 #endif /* GEANY_PRIVATE */
 
-#ifdef __cplusplus
-}
-#endif
+G_END_DECLS
 
 #endif /* TM_TAG_H */
